@@ -23,6 +23,44 @@ XSAN_CONFIG_NODE_REGEX = r'Node \"(.+)\" \d+'
 CLIENT_UPDATE_DELAY = 2.0
 
 
+class process_launcher():
+    """Launches multiple PROCESSES.
+    Watches if they're alive.
+    If any of them exits, terminates the whole bunch
+
+    """
+    PROCESSES = []
+    ALL_PROCESSES_ALIVE = True
+
+    def add_process(self, process):
+        self.PROCESSES.append(process)
+
+    def run(self):
+        for process in self.PROCESSES:
+            process.start()
+
+        while True:
+            #   Check if all processes are alive
+            for process in self.PROCESSES:
+                process.join(0.5)
+                if not process.is_alive():
+                    LOGGER.info(
+                        "Process %s had unhandled exception, exiting",
+                        process.name)
+                    self.ALL_PROCESSES_ALIVE = False
+
+            if not self.ALL_PROCESSES_ALIVE:
+                #   Kill everyone and exit
+                for process in self.PROCESSES:
+                    LOGGER.info(
+                        "Killing process %s",
+                        process.name)
+                    process.terminate()
+
+                #   Exit the loop, terminate the program
+                break
+
+
 def get_xsan_config_dir():
     """Gets Xsan config directory path based on the OS version
 
@@ -313,20 +351,27 @@ def update_clients():
     global XSAN_CONFIG_NODE_REGEX
     global CLIENT_UPDATE_DELAY
 
-    processes = []
+    while True:
+        processes = []
 
-    for client in models.xsanio_client.objects.all():
-        LOGGER.info('Getting stats for %s', client)
-        processes.append(multiprocessing.Process(
-            target=get_stats_from_client,
-            args=(client, CLIENT_UPDATE_DELAY, )
-        ))
+        for client in models.xsanio_client.objects.all():
+            LOGGER.info('Getting stats for %s', client)
+            processes.append(multiprocessing.Process(
+                target=get_stats_from_client,
+                args=(client, CLIENT_UPDATE_DELAY, )
+            ))
 
-    for process in processes:
-        process.start()
+        for process in processes:
+            process.start()
 
-    for process in processes:
-        process.join()
+        for process in processes:
+            process.join(0.5)
+            if not process.is_alive():
+                LOGGER.error('Process crashed, restarting everyone')
+                break
+
+        for process in processes:
+            process.terminate()
 
 
 def main():
